# ----------------------------------------------------------------------------
#
#  Welcome to Baml! To use this generated code, please run the following:
#
#  $ pip install baml
#
# ----------------------------------------------------------------------------

# This file was generated by BAML: please do not edit it. Instead, edit the
# BAML files and re-generate this code using: baml-cli generate
# baml-cli is available with the baml package.

from . import stream_types, types

type_map = {
    "types.ChunkInput": types.ChunkInput,
    "stream_types.ChunkInput": stream_types.ChunkInput,
    "types.ChunkSummary": types.ChunkSummary,
    "stream_types.ChunkSummary": stream_types.ChunkSummary,
    "types.FileSummaryResult": types.FileSummaryResult,
    "stream_types.FileSummaryResult": stream_types.FileSummaryResult,
}
