# ----------------------------------------------------------------------------
#
#  Welcome to Baml! To use this generated code, please run the following:
#
#  $ pip install baml
#
# ----------------------------------------------------------------------------

# This file was generated by BAML: please do not edit it. Instead, edit the
# BAML files and re-generate this code using: baml-cli generate
# baml-cli is available with the baml package.

__version__ = "2.14.9"

try:
    from baml_py.safe_import import EnsureBamlPyImport
except ImportError:
    raise ImportError(f"""Update to baml-py required.
Version of baml_client generator (see generators.baml): {__version__}

Please upgrade baml-py to version "{__version__}".

$ pip install baml-py=={__version__}
$ uv add baml-py=={__version__}

If nothing else works, please ask for help:

https://github.com/boundaryml/baml/issues
https://boundaryml.com/discord
""") from None


with EnsureBamlPyImport(__version__) as e:
    e.raise_if_incompatible_version(__version__)

    from . import config, stream_types, tracing, types, watchers
    from .async_client import b
    from .config import reset_baml_env_vars


# FOR LEGACY COMPATIBILITY, expose "partial_types" as an alias for "stream_types"
# WE RECOMMEND USERS TO USE "stream_types" INSTEAD
partial_types = stream_types

__all__ = [
    "b",
    "stream_types",
    "partial_types",
    "tracing",
    "types",
    "reset_baml_env_vars",
    "config",
    "watchers",
]
